﻿using Kivii.Auth;
using Kivii.Finances.Entities;
using Kivii.Finances.Seeyon.Entities;
using Kivii.Seeyon.Entities;
using Kivii.Text;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon.Extensions
{
    internal static class ApplyExtension
    {
        public static ResponseToken RequestRestToken(this JsonServiceClient client, string loginName)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (loginName.IsNullOrEmpty()) throw new ArgumentNullException("loginName");

            try
            {
                var request = new RequestRestToken();
                request.userName = Configs.restUserName;
                request.password = Configs.restPassword;
                request.loginName = loginName;
                var rtns = client.Post<ResponseToken>($"{Configs.RouteRestToken}", request);
                return rtns;
            }
            catch (Exception ex)
            {
                throw new Exception($"{ex.Message}，可能原因：当前登录人{loginName}，无法与OA账号对应，请联系管理员确认！");
            }
        }

        public static ResponseOrgMember RequestOrgMember(this JsonServiceClient client, string loginName, ResponseToken token)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (loginName.IsNullOrEmpty()) throw new ArgumentNullException("loginName");

            try
            {
                var rtns = client.Get<ResponseOrgMember>($"{Configs.RouteOrgMember}?token={token.id}&&loginName={loginName}");
                return rtns;
            }
            catch (Exception ex)
            {
                throw new Exception($"{ex.Message}，可能原因：当前登录人{loginName}，无法与OA账号对应，请联系管理员确认！");
            }
        }

        public static ResponseForm<ResponseData> RequestBillForm0710(this JsonServiceClient client, FormMain_OrdinaryInvoice0710 form, ResponseToken token)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (form == null) throw new ArgumentNullException("form");
            //var token = client.RequestRestToken(session.FullName);
            var request = new RequestForm<RequestForm_InvoiceApply>();
            request.appName = Configs.appName;//"collaboration";
            request.data = new RequestData<RequestForm_InvoiceApply>();
            request.data.templateCode = Configs.templateCodeOrdinaryInvoice;//"JYHTSQ";
            var templateCodeSuffix = form.收款单位名称.GetTemplateCodeSuffix();
            if (!templateCodeSuffix.IsNullOrEmpty()) request.data.templateCode = $"{Configs.templateCodeOrdinaryInvoice}_{templateCodeSuffix}";
            request.data.draft = "0";
            //request.data.senderLoginName = token.bindingUser.loginName;
            request.data.subject = $"到账发票申请({token.bindingUser.name} {DateTime.Now:yyyy-MM-dd HH:mm})";
            request.data.data = new RequestForm_InvoiceApply();
            request.data.data.formmain_0710 = new FormMain_OrdinaryInvoice0710();
            request.data.data.formmain_0710 = form;
            Console.WriteLine("-------------------------------------------------------------------------");
            Console.WriteLine(JsonSerializer.SerializeToString(request));
            Console.WriteLine("-------------------------------------------------------------------------");
            var rtns = client.Post<ResponseForm<ResponseData>>($"{Configs.RouteRequestForm}?token={token.id}", request);
            return rtns;
        }

        public static ResponseForm<ResponseData> RequestBillForm0318(this JsonServiceClient client, FormMain_AdvanceInvoice0318 form, ResponseToken token)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (form == null) throw new ArgumentNullException("form");
            //var token = client.RequestRestToken(session.FullName);
            var request = new RequestForm<RequestForm_InvoiceApply>();
            request.appName = Configs.appName;//"collaboration";
            request.data = new RequestData<RequestForm_InvoiceApply>();
            request.data.templateCode = Configs.templateCodeAdvanceInvoice;//"JYHTSQ";
            var templateCodeSuffix = form.收款单位.GetTemplateCodeSuffix();
            if (!templateCodeSuffix.IsNullOrEmpty()) request.data.templateCode = $"{Configs.templateCodeAdvanceInvoice}_{templateCodeSuffix}";
            request.data.draft = "0";
            //request.data.senderLoginName = token.bindingUser.loginName;
            request.data.subject = $"预借发票申请({token.bindingUser.name} {DateTime.Now:yyyy-MM-dd HH:mm})";
            request.data.data = new RequestForm_InvoiceApply();
            request.data.data.formmain_0318 = new FormMain_AdvanceInvoice0318();
            request.data.data.formmain_0318 = form;
            Console.WriteLine("-------------------------------------------------------------------------");
            Console.WriteLine(JsonSerializer.SerializeToString(request));
            Console.WriteLine("-------------------------------------------------------------------------");
            var rtns = client.Post<ResponseForm<ResponseData>>($"{Configs.RouteRequestForm}?token={token.id}", request);
            return rtns;
        }


        public static FormMain_OrdinaryInvoice0710 PayedConvertToForm(this List<InvoiceApply> applys, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var apply = applys[0];
            var bizKvids = applys.ConvertAll(o => o.Kvid.ToString());
            //var strBizKvids = JsonSerializer.SerializeToString(bizKvids);
            var amount = applys.Sum(o => o.Amount);
            var rtns = new FormMain_OrdinaryInvoice0710();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(BillApply).FullName;
            rtns.业务编号 = bizKvids;
            rtns.单据编号 = apply.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.单位 = orgMember.orgAccountId;
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : (apply.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.数量 = "1";
            rtns.总金额 = amount.ToString();
            rtns.金额大写 = amount.ConvertToChinese();
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.收款单位名称 = apply.PayeeName;
            rtns.收款单位纳税人识别号 = apply.PayeeTaxNumber;
            rtns.收款单位地址电话 = $"{apply.PayeeCompanyAddress}{apply.PayeePhone}";
            rtns.收款单位开户行及账号 = $"{apply.PayeeRegisteredBank}{apply.PayeeBankAccount}";
            rtns.申请原因 = apply.Remark;
            rtns.收款单位名称 = apply.PayeeName;
            return rtns;
        }

        public static FormMain_OrdinaryInvoice0710 PayedConvertToForm(this InvoiceApply apply, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var bizKvids = new List<string>();
            bizKvids.Add(apply.Kvid.ToString());
            var rtns = new FormMain_OrdinaryInvoice0710();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(BillApply).FullName;
            rtns.业务编号 = bizKvids;
            rtns.单据编号 = apply.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.单位 = orgMember.orgAccountId;
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : (apply.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.数量 = "1";
            rtns.总金额 = apply.Amount.ToString();
            rtns.金额大写 = apply.Amount.ConvertToChinese();
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.收款单位名称 = apply.PayeeName;
            rtns.收款单位纳税人识别号 = apply.PayeeTaxNumber;
            rtns.收款单位地址电话 = $"{apply.PayeeCompanyAddress}{apply.PayeePhone}";
            rtns.收款单位开户行及账号 = $"{apply.PayeeRegisteredBank}{apply.PayeeBankAccount}";
            rtns.申请原因 = apply.Remark;
            rtns.收款单位名称 = apply.PayeeName;
            return rtns;
        }

        public static FormMain_AdvanceInvoice0318 DebitConvertToForm(this List<InvoiceApply> applys, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var apply = applys[0];
            var bizKvids = applys.ConvertAll(o => o.Kvid.ToString());
            //var strBizKvids = JsonSerializer.SerializeToString(bizKvids);
            var amount = applys.Sum(o => o.Amount);
            var rtns = new FormMain_AdvanceInvoice0318();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(BillApply).FullName;
            rtns.业务编号 = bizKvids;
            rtns.单据编号 = apply.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.单位 = orgMember.orgAccountId;
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : (apply.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.数量 = "1";
            rtns.总金额 = amount.ToString();
            rtns.金额大写 = amount.ConvertToChinese();
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.收款单位 = apply.PayeeName;
            rtns.收款单位纳税人识别号 = apply.PayeeTaxNumber;
            rtns.收款单位地址电话 = $"{apply.PayeeCompanyAddress}{apply.PayeePhone}";
            rtns.收款单位开户行及账号 = $"{apply.PayeeRegisteredBank}{apply.PayeeBankAccount}";
            rtns.申请原因 = apply.Remark;
            rtns.收款单位 = apply.PayeeName;
            return rtns;
        }

        public static FormMain_AdvanceInvoice0318 DebitConvertToForm(this InvoiceApply apply, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var bizKvids = new List<string>();
            bizKvids.Add(apply.Kvid.ToString());
            var rtns = new FormMain_AdvanceInvoice0318();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(BillApply).FullName;
            rtns.业务编号 = bizKvids;
            rtns.单据编号 = apply.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.单位 = orgMember.orgAccountId;
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : (apply.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.数量 = "1";
            rtns.总金额 = apply.Amount.ToString();
            rtns.金额大写 = apply.Amount.ConvertToChinese();
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.收款单位 = apply.PayeeName;
            rtns.收款单位纳税人识别号 = apply.PayeeTaxNumber;
            rtns.收款单位地址电话 = $"{apply.PayeeCompanyAddress}{apply.PayeePhone}";
            rtns.收款单位开户行及账号 = $"{apply.PayeeRegisteredBank}{apply.PayeeBankAccount}";
            rtns.申请原因 = apply.Remark;
            rtns.收款单位 = apply.PayeeName;
            return rtns;
        }

        public static string GetTemplateCodeSuffix(this string payeeName)
        {
            var rtns = string.Empty;
            if (payeeName.Contains("南京市产品质量监督检验院")) rtns = "ZJY";
            if (payeeName.Contains("深圳市宁深检验检测技术有限公司")) rtns = "NS";
            if (payeeName.Contains("江苏苏测智能装备检测有限公司")) rtns = "ZB";
            if (payeeName.Contains("江苏苏测检测认证有限公司")) rtns = "SC";
            return rtns;
        }

        public static string ConvertToChinese(this decimal Num)
        {
            string[] DX_SZ = { "零", "壹", "贰", "叁", "肆", "伍", "陆", "柒", "捌", "玖", "拾" };//大写数字  
            string[] DX_DW = { "元", "拾", "佰", "仟", "万", "拾", "佰", "仟", "亿", "拾", "佰", "仟", "万" };
            string[] DX_XSDS = { "角", "分" };//大些小数单位  
            if (Num == 0) return DX_SZ[0];

            Boolean IsXS_bool = false;//是否小数  

            string NumStr;//整个数字字符串  
            string NumStr_Zs;//整数部分  
            string NumSr_Xs = "";//小数部分  
            string NumStr_R = "";//返回的字符串  


            NumStr = Num.ToString();
            NumStr_Zs = NumStr;
            if (NumStr_Zs.Contains("."))
            {
                NumStr = Math.Round(Num, 2).ToString();
                NumStr_Zs = NumStr.Substring(0, NumStr.IndexOf("."));
                NumSr_Xs = NumStr.Substring((NumStr.IndexOf(".") + 1), (NumStr.Length - NumStr.IndexOf(".") - 1));
                IsXS_bool = true;
            }

            int k = 0;
            Boolean IsZeor = false;//整数中间连续0的情况  
            for (int i = 0; i < NumStr_Zs.Length; i++) //整数  
            {
                int j = int.Parse(NumStr_Zs.Substring(i, 1));
                if (j != 0)
                {
                    NumStr_R += DX_SZ[j] + DX_DW[NumStr_Zs.Length - i - 1];
                    IsZeor = false; //没有连续0  
                }
                else if (j == 0)
                {
                    k++;
                    if (!IsZeor && !(NumStr_Zs.Length == i + 1)) //等于0不是最后一位，连续0取一次  
                    {
                        //有问题  
                        if (NumStr_Zs.Length - i - 1 >= 4 && NumStr_Zs.Length - i - 1 <= 6)
                            NumStr_R += DX_DW[4] + "零";
                        else
                            if (NumStr_Zs.Length - i - 1 > 7)
                            NumStr_R += DX_DW[8] + "零";
                        else
                            NumStr_R += "零";

                        IsZeor = true;
                    }

                    if (NumStr_Zs.Length == i + 1)//  等于0且是最后一位 变成 XX元整  
                        NumStr_R += DX_DW[NumStr_Zs.Length - i - 1];
                }

            }
            if (NumStr_Zs.Length > 2 && k == NumStr_Zs.Length - 1)
                NumStr_R = NumStr_R.Remove(NumStr_R.IndexOf('零'), 1); //比如1000，10000元整的情况下 去0  

            if (!IsXS_bool) return NumStr_R + "整"; //如果没有小数就返回  
            else
            {
                for (int i = 0; i < NumSr_Xs.Length; i++)
                {
                    int j = int.Parse(NumSr_Xs.Substring(i, 1));
                    NumStr_R += DX_SZ[j] + DX_XSDS[NumSr_Xs.Length - i - 1];
                }
            }

            return NumStr_R;
        }
    }
}
