﻿using Kivii.DataAnnotations;
using Kivii.Finances.Entities;
using Kivii.Finances.Seeyon.Extensions;
using Kivii.Linq;
using Kivii.Seeyon;
using Kivii.Seeyon.Entities;
using Kivii.Text;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon.Entities
{
    [Api(Description = "账单申请")]
    [Alias(Configs.TableNameApply)]
    public class BillApply : InvoiceApply, IEntityIsSeeyonApply
    {
        [Ignore]
        public List<Guid> Kvids { get; set; }

        public object OnGetApplyResult(Guid Kvid, object Result, string Message)
        {
            if (Kvid == Guid.Empty) throw new Exception("缺少业务编号!");
            if (Result == null) throw new Exception("缺少审批结果信息");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var rtns = new RestfulExecutionResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();
            var apply = conn.SingleById<InvoiceApply>(Kvid);
            if (apply == null) throw new Exception("未找到此业务信息!");
            var result = Result.ToString();
            if (apply.Metadata.IsNullOrEmpty()) apply.Metadata = new Dictionary<string, string>();
            EntityLog<InvoiceApply> log = null;
            if (result == "审批通过")
            {
                apply.Metadata["SeeyonResult"] = result;
                apply.AddOnlyProperties(o => o.Metadata);
                if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                {
                    apply.Status = (int)InvoiceApplyStatus.FinancialApproval;
                    apply.AddOnlyProperties(o => o.Status);
                    log = new EntityLog<InvoiceApply>();
                    log.OwnerKvid = apply.Kvid;
                    log.Title = "账单借票";
                    log.Type = "OA申请";
                    log.Summary = "OA审批通过,可进行开票";
                    log.Remark = Message;
                    log.CreatorName = "OA审批";
                }
            }
            else if (result == "审批不通过")
            {
                apply.Metadata["SeeyonResult"] = result;
                apply.Metadata["SeeyonMessage"] = Message;
                apply.AddOnlyProperties(o => o.Metadata);
                if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                {
                    apply.Status = (int)InvoiceApplyStatus.ApplyReject;
                    apply.AddOnlyProperties(o => o.Status);
                    log = new EntityLog<InvoiceApply>();
                    log.OwnerKvid = apply.Kvid;
                    log.Title = "账单借票";
                    log.Type = "OA申请";
                    log.Summary = "OA审批不通过,驳回了账单借票审批";
                    log.Remark = Message;
                    log.CreatorName = "OA审批";
                }
            }
            if (log != null) conn.Insert(log);
            if (!apply.OnlyProperties.IsNullOrEmpty())
            {
                conn.UpdateOnly(apply);
                rtns.Results.Add(apply);
            }

            return rtns;
        }

        public object OnGetApplyResult(List<Guid> Kvids, object Result, string Message)
        {
            if (Kvids.IsNullOrEmpty()) throw new Exception("缺少业务编号!");
            if (Result == null) throw new Exception("缺少审批结果信息");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var rtns = new RestfulExecutionResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();
            var applys = conn.SelectByIds<InvoiceApply>(Kvids);
            if (applys.IsNullOrEmpty()) throw new Exception("未找到此业务信息!");
            //var apply = conn.SingleById<InvoiceApply>(Kvids);
            //if (apply == null) throw new Exception("未找到此业务信息!");
            var result = Result.ToString();
            foreach (var apply in applys)
            {
                if (apply.Metadata.IsNullOrEmpty()) apply.Metadata = new Dictionary<string, string>();
                EntityLog<InvoiceApply> log = null;
                if (result == "审批通过")
                {
                    apply.Metadata["SeeyonResult"] = result;
                    apply.AddOnlyProperties(o => o.Metadata);
                    if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                    {
                        apply.Status = (int)InvoiceApplyStatus.FinancialApproval;
                        apply.AddOnlyProperties(o => o.Status);
                        log = new EntityLog<InvoiceApply>();
                        log.OwnerKvid = apply.Kvid;
                        log.Title = "账单借票";
                        log.Type = "OA申请";
                        log.Summary = "OA审批通过,可进行开票";
                        log.Remark = Message;
                        log.CreatorName = "OA审批";
                    }
                }
                else if (result == "审批不通过")
                {
                    apply.Metadata["SeeyonResult"] = result;
                    apply.Metadata["SeeyonMessage"] = Message;
                    apply.AddOnlyProperties(o => o.Metadata);
                    if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                    {
                        apply.Status = (int)InvoiceApplyStatus.ApplyReject;
                        apply.AddOnlyProperties(o => o.Status);
                        log = new EntityLog<InvoiceApply>();
                        log.OwnerKvid = apply.Kvid;
                        log.Title = "账单借票";
                        log.Type = "OA申请";
                        log.Summary = "OA审批不通过,驳回了账单借票审批";
                        log.Remark = Message;
                        log.CreatorName = "OA审批";
                    }
                }
                if (log != null) conn.Insert(log);
                if (!apply.OnlyProperties.IsNullOrEmpty())
                {
                    conn.UpdateOnly(apply);
                    rtns.Results.Add(apply);
                }
            }
            return rtns;
        }

        public object OnSubmitApplyForm<T>(Guid Kvid)
        {
            Kvid.ThrowIfEmpty("请传入要申请的账单借票信息!");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            InvoiceApply apply = conn.SingleById<InvoiceApply>(Kvid);
            apply.ThrowIfNull("未找到此业务信息!");
            if (apply.Status > (int)InvoiceApplyStatus.ProcessAdoption) throw new Exception("当前账单借票非待审批状态，无法申请审批！");
            var session = KiviiContext.Request.GetSession();
            var client = new JsonServiceClient(Configs.BaseUrl);
            var token = client.RequestRestToken(session.FullName);
            var orgMember = client.RequestOrgMember(session.FullName, token);

            var rtns = new RestfulUpdateResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();

            if (apply.OperateType == InvoiceApplyType.Payment)
            {
                var applyForm = apply.PayedConvertToForm(orgMember);
                var resp = client.RequestBillForm0710(applyForm, token);
                if (resp.code != "0") throw new Exception($"OA接口调用失败,原因:{resp.message}");
                apply.BizId = resp.data.subject;
                apply.AddOnlyProperties(o => o.BizId);
                apply.BizType = typeof(BillApply).FullName;
                apply.AddOnlyProperties(o => o.BizType);
                apply.Status = (int)InvoiceApplyStatus.ProcessAdoption;
                apply.AddOnlyProperties(o => o.Status);
                conn.UpdateOnly(apply);
                var _log = new EntityLog<InvoiceApply>();
                _log.OwnerKvid = apply.Kvid;
                _log.Title = "账单到账开票申请";
                _log.Type = "OA申请";
                _log.Summary = $"提交了账单到账开票申请,审批将在OA执行";
                conn.Insert(_log);

                rtns.Results.Add(apply);
            }
            else
            {
                var applyForm = apply.DebitConvertToForm(orgMember);
                var resp = client.RequestBillForm0318(applyForm, token);
                if (resp.code != "0") throw new Exception($"OA接口调用失败,原因:{resp.message}");
                apply.BizId = resp.data.subject;
                apply.AddOnlyProperties(o => o.BizId);
                apply.BizType = typeof(BillApply).FullName;
                apply.AddOnlyProperties(o => o.BizType);
                apply.Status = (int)InvoiceApplyStatus.ProcessAdoption;
                apply.AddOnlyProperties(o => o.Status);
                conn.UpdateOnly(apply);
                var _log = new EntityLog<InvoiceApply>();
                _log.OwnerKvid = apply.Kvid;
                _log.Title = "账单借票申请";
                _log.Type = "OA申请";
                _log.Summary = $"提交了账单借票申请,审批将在OA执行";
                conn.Insert(_log);

                rtns.Results.Add(apply);
            }


            return rtns;
        }

        public object OnSubmitApplyForm<T>(T form)
        {
            form.ThrowIfNull("请传入借票信息！");
            var formApply = form as BillApply;
            formApply.ThrowIfNull("传入的借票信息不能为空！");
            var kvids = formApply.Kvids;
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var applys = conn.SelectByIds<InvoiceApply>(kvids);
            applys.ThrowIfNullOrEmpty("未找到借票信息!");
            if (applys.Exists(o => o.Status > (int)InvoiceApplyStatus.ProcessAdoption)) throw new Exception("当前账单借票非待审批状态，无法申请审批！");

            var groupPayer = applys.GroupBy(a => a.PayerName);
            if (groupPayer.Count() != 1) throw new Exception("不允许多个借票抬头同时申请！");
            var groupType = applys.GroupBy(o => o.OperateType);
            if (groupType.Count() != 1) throw new Exception("不允许多个申请类型同时申请！");
            var session = KiviiContext.Request.GetSession();
            var client = new JsonServiceClient(Configs.BaseUrl);
            var token = client.RequestRestToken(session.FullName);
            var orgMember = client.RequestOrgMember(session.FullName, token);

            var rtns = new RestfulUpdateResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();

            if (groupType.FirstOrDefault().Key == InvoiceApplyType.Payment)
            {
                var applyForm = applys.PayedConvertToForm(orgMember);
                var resp = client.RequestBillForm0710(applyForm, token);
                if (resp.code != "0") throw new Exception($"OA接口调用失败,原因:{resp.message}");
                foreach (var apply in applys)
                {
                    apply.BizId = resp.data.subject;
                    apply.AddOnlyProperties(o => o.BizId);
                    apply.BizType = typeof(BillApply).FullName;
                    apply.AddOnlyProperties(o => o.BizType);
                    apply.Status = (int)InvoiceApplyStatus.ProcessAdoption;
                    apply.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(apply);
                    var _log = new EntityLog<InvoiceApply>();
                    _log.OwnerKvid = apply.Kvid;
                    _log.Title = "账单到账开票申请";
                    _log.Type = "OA申请";
                    _log.Summary = $"提交了账单到账开票申请,审批将在OA执行";
                    conn.Insert(_log);

                    rtns.Results.Add(apply);
                }
            }
            else
            {
                var applyForm = applys.DebitConvertToForm(orgMember);
                var resp = client.RequestBillForm0318(applyForm, token);
                if (resp.code != "0") throw new Exception($"OA接口调用失败,原因:{resp.message}");
                foreach (var apply in applys)
                {
                    apply.BizId = resp.data.subject;
                    apply.AddOnlyProperties(o => o.BizId);
                    apply.BizType = typeof(BillApply).FullName;
                    apply.AddOnlyProperties(o => o.BizType);
                    apply.Status = (int)InvoiceApplyStatus.ProcessAdoption;
                    apply.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(apply);
                    var _log = new EntityLog<InvoiceApply>();
                    _log.OwnerKvid = apply.Kvid;
                    _log.Title = "账单借票申请";
                    _log.Type = "OA申请";
                    _log.Summary = $"提交了账单借票申请,审批将在OA执行";
                    conn.Insert(_log);

                    rtns.Results.Add(apply);
                }
            }
            return rtns;
        }
    }

    public class RequestForm_InvoiceApply
    {
        public FormMain_OrdinaryInvoice0710 formmain_0710 { get; set; }

        public FormMain_AdvanceInvoice0318 formmain_0318 { get; set; }
    }

    #region OA表单
    /// <summary>
    /// 发起流程表单接口【到账发票申请】
    /// </summary>
    public class FormMain_OrdinaryInvoice0710
    {
        /// <summary>
        /// 唯一编号,BizKvid
        /// </summary>
        public List<string> 业务编号 { get; set; }

        /// <summary>
        /// 业务类型,BizType
        /// </summary>
        public string 业务类型 { get; set; }
        /// <summary>
        /// 单据编号
        /// </summary>
        public string 单据编号 { get; set; }

        /// <summary>
        /// 格式：2023-10-11
        /// </summary>
        public string 申请日期 { get; set; }

        /// <summary>
        /// 申请人（传人员ID）
        /// </summary>
        public string 申请人 { get; set; }

        /// <summary>
        /// 单位
        /// </summary>
        public string 单位 { get; set; }

        /// <summary>
        /// 申请部门（传申请部门ID）
        /// </summary>
        public string 申请部门 { get; set; }

        public string 收款单位名称 { get; set; }

        public string 收款单位纳税人识别号 { get; set; }

        public string 收款单位地址电话 { get; set; }

        public string 收款单位开户行及账号 { get; set; }
        /// <summary>
        /// 发票类型{普通：4529935671729733794  专票：-4189929216282050282  收据：4728848026009816823}
        /// </summary>
        public string 发票类型 { get; set; }

        public string 数量 { get; set; }

        public string 总金额 { get; set; }

        public string 金额大写 { get; set; }

        public string 付款单位名称 { get; set; }

        public string 纳税人识别号 { get; set; }

        public string 地址电话 { get; set; }

        public string 开户行及账号 { get; set; }

        public string 申请原因 { get; set; }

    }

    /// <summary>
    /// 发起流程表单接口【预借发票/收据申请】
    /// </summary>
    public class FormMain_AdvanceInvoice0318
    {
        /// <summary>
        /// 唯一编号,BizKvid
        /// </summary>
        public List<string> 业务编号 { get; set; }

        /// <summary>
        /// 业务类型,BizType
        /// </summary>
        public string 业务类型 { get; set; }
        /// <summary>
        /// 单据编号
        /// </summary>
        public string 单据编号 { get; set; }

        /// <summary>
        /// 格式：2023-10-11
        /// </summary>
        public string 申请日期 { get; set; }

        /// <summary>
        /// 申请人（传人员ID）
        /// </summary>
        public string 申请人 { get; set; }

        /// <summary>
        /// 申请部门（传申请部门）
        /// </summary>
        public string 申请部门 { get; set; }

        public string 单位 { get; set; }

        public string 付款单位名称 { get; set; }

        public string 纳税人识别号 { get; set; }

        public string 地址电话 { get; set; }

        public string 开户行及账号 { get; set; }

        /// <summary>
        /// 发票类型{普通：4529935671729733794  专票：-4189929216282050282  收据：4728848026009816823}
        /// </summary>
        public string 发票类型 { get; set; }

        public string 申请原因 { get; set; }

        public string 数量 { get; set; }

        public string 总金额 { get; set; }

        public string 收款单位 { get; set; }

        public string 收款单位纳税人识别号 { get; set; }

        public string 收款单位地址电话 { get; set; }

        public string 收款单位开户行及账号 { get; set; }

        public string 金额大写 { get; set; }

    }

    /// <summary>
    /// 发起流程表单接口【发票当月作废申请】
    /// </summary>
    public class FormMain_InvoiceVoid0712
    {
        /// <summary>
        /// 唯一编号,BizKvid
        /// </summary>
        public List<string> 业务编号 { get; set; }

        /// <summary>
        /// 业务类型,BizType
        /// </summary>
        public string 业务类型 { get; set; }
        /// <summary>
        /// 单据编号
        /// </summary>
        public string 单据编号 { get; set; }

        /// <summary>
        /// 格式：2023-10-11
        /// </summary>
        public string 申请日期 { get; set; }

        /// <summary>
        /// 申请部门（传部门ID）
        /// </summary>
        public string 申请部门 { get; set; }

        /// <summary>
        /// 申请人（传人员ID）
        /// </summary>
        public string 申请人 { get; set; }

        public string 原发票号 { get; set; }

        /// <summary>
        /// 发票类型{普通：4529935671729733794  专票：-4189929216282050282  收据：4728848026009816823}
        /// </summary>
        public string 原发票类型 { get; set; }

        public string 原发票开票单位 { get; set; }

        public string 原因 { get; set; }

        public string 原开票金额 { get; set; }
    }

    /// <summary>
    /// 发起流程表单接口【跨月发票红冲申请】
    /// </summary>
    public class FormMain_InvoiceRedOffset0338
    {
        /// <summary>
        /// 唯一编号,BizKvid
        /// </summary>
        public List<string> 业务编号 { get; set; }

        /// <summary>
        /// 业务类型,BizType
        /// </summary>
        public string 业务类型 { get; set; }
        /// <summary>
        /// 单据编号
        /// </summary>
        public string 单据编号 { get; set; }

        /// <summary>
        /// 格式：2023-10-11
        /// </summary>
        public string 申请日期 { get; set; }

        /// <summary>
        /// 申请部门（传部门ID）
        /// </summary>
        public string 申请部门 { get; set; }

        /// <summary>
        /// 申请人（传人员ID）
        /// </summary>
        public string 申请人 { get; set; }

        /// <summary>
        /// 是：-9179045864512534208  否：-7622688603542042575
        /// </summary>
        public string 是否为预借发票 { get; set; }

        public string 原因 { get; set; }

        public string 原发票号 { get; set; }

        /// <summary>
        /// 发票类型{普通：4529935671729733794  专票：-4189929216282050282  收据：4728848026009816823}
        /// </summary>
        public string 原发票类型 { get; set; }

        public string 原发票开票单位 { get; set; }


        public string 原开票金额 { get; set; }

        public string 冲红发票号码 { get; set; }
    }
    #endregion
}
