﻿using Kivii.Finances.Entities;
using Kivii.Finances.Seeyon.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon
{
    [Api(Description = "OA发票回写调用")]
    public class BillApplyResult: RestfulExecution<BillApply>
    {
        public Guid Kvid { get; set; }//InvoiceApplyKvid

        public List<Guid> Kvids { get; set; }//InvoiceApplyKvids

        public string Result { get; set; }//  审批通过/审批不通过

        public List<Invoice> Invoices { get; set; }// 发票信息，最少要有发票号码和开票时间

        public string Message { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()) throw new Exception("业务编号不能为空！");
            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);
            if(Result.IsNullOrEmpty()) throw new Exception("请传入审批结果：【审批通过/审批不通过】！");

            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var applys = conn.SelectByIds<InvoiceApply>(Kvids);
            if (applys.IsNullOrEmpty()) throw new Exception("未找到指定的申请信息!");
            if (applys.Exists(o => o.Status > (int)InvoiceApplyStatus.AutoExecuting)) throw new Exception("申请信息状态发生变动，无法回写！");

            var rtns = new RestfulExecutionResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();

            if (Result == "审批通过")
            {
                if (Invoices.IsNullOrEmpty()) throw new Exception("请填写发票信息");
                var trans = conn.OpenTransaction();
                try
                {
                    if (Invoices.IsNullOrEmpty())
                    {
                        foreach (var apply in applys)
                        {
                            if (apply.Metadata.IsNullOrEmpty()) apply.Metadata = new Dictionary<string, string>();
                            apply.Metadata["SeeyonResult"] = Result;
                            apply.Metadata["SeeyonMessage"] = Message;
                            apply.AddOnlyProperties(o => o.Metadata);
                            apply.Status = (int)InvoiceApplyStatus.FinancialApproval;
                            apply.AddOnlyProperties(o => o.Status);
                            conn.UpdateOnly(apply);
                            rtns.Results.Add(apply);
                            var log = new EntityLog<InvoiceApply>();
                            log.OwnerKvid = apply.Kvid;
                            log.Title = "账单借票";
                            log.Type = "OA申请";
                            log.Summary = "OA审批通过,可进行开票";
                            log.Remark = Message;
                            log.CreatorName = "OA审批";
                            conn.Insert(log);
                        }
                    }//回写没传入发票信息，那就更新对应申请状态
                    else if (applys.Count != 1)
                    {
                        foreach (var apply in applys)
                        {
                            if (apply.Metadata.IsNullOrEmpty()) apply.Metadata = new Dictionary<string, string>();
                            apply.Metadata["SeeyonResult"] = Result;
                            apply.Metadata["SeeyonMessage"] = Message;
                            apply.AddOnlyProperties(o => o.Metadata);
                            apply.Status = (int)InvoiceApplyStatus.FinancialApproval;
                            apply.AddOnlyProperties(o => o.Status);
                            conn.UpdateOnly(apply);
                            rtns.Results.Add(apply);
                            var log = new EntityLog<InvoiceApply>();
                            log.OwnerKvid = apply.Kvid;
                            log.Title = "账单借票";
                            log.Type = "OA申请";
                            log.Summary = "OA审批通过,可进行开票";
                            log.Remark = Message;
                            log.CreatorName = "OA审批";
                            conn.Insert(log);
                        }
                    }//如果申请超过1条，那也不做处理，只更新申请状态
                    else//只有一个申请的时候，那就把对应的发票记录下来
                    {
                        //var logs = new List<EntityLog<InvoiceApply>>();
                        var updateApplys = new List<InvoiceApply>();
                        var insertInvoices = new List<Invoice>();
                        var insertInvoiceDetails = new List<InvoiceDetail>();

                        var apply = applys[0];
                        (apply.Amount - apply.AmountInvoice != Invoices.Sum(o => o.Amount)).ThrowIfTrue("当前申请金额不等于要录入的发票金额!");
                        var applyRelations = conn.Select<InvoiceApply>(o => o.ParentKvid == apply.Kvid && o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty);
                        foreach (var info in Invoices)
                        {
                            #region 开票成功的情况处理
                            var time = info.OperateTime == DateTime.MinValue ? DateTime.Now : info.OperateTime;//开票时间
                            var number = info.SerialNumber;//开票号码,唯一
                            var type = apply.Type;//开票类型
                            var currentAmount = info.Amount;
                            var taxRate = 0.03m;
                            var invoiceKvid = Guid.NewGuid();
                            var invoice = new Invoice();

                            invoice.RootKvid = invoiceKvid;
                            invoice.Kvid = invoiceKvid;
                            invoice.ApplyKvid = apply.Kvid;
                            invoice.OwnerKvid = apply.OwnerKvid;
                            invoice.OwnerName = apply.OwnerName;
                            invoice.Currency = CurrencyUnit.CNY;
                            invoice.Type = type;
                            invoice.Category = apply.OperateType.ToString();
                            invoice.OperateTime = time;
                            invoice.SerialNumber = number;
                            invoice.Amount = currentAmount;
                            invoice.AmountUntaxed = Math.Round(currentAmount / (1 + taxRate), 2);
                            invoice.AmountTax = currentAmount - invoice.AmountUntaxed;
                            invoice.TaxRate = taxRate;
                            invoice.PayeeKvid = apply.PayeeKvid;
                            invoice.PayeeName = apply.PayeeName;
                            invoice.PayeeTaxNumber = apply.PayeeTaxNumber;
                            invoice.PayeeCompanyAddress = apply.PayeeCompanyAddress;
                            invoice.PayeePhone = apply.PayeePhone;
                            invoice.PayeeRegisteredBank = apply.PayeeRegisteredBank;
                            invoice.PayeeBankAccount = apply.PayeeBankAccount;
                            invoice.PayerKvid = apply.PayerKvid;
                            invoice.PayerName = apply.PayerName;
                            invoice.PayerTaxNumber = apply.PayerTaxNumber;
                            invoice.PayerCompanyAddress = apply.PayerCompanyAddress;
                            invoice.PayerPhone = apply.PayerPhone;
                            invoice.PayerRegisteredBank = apply.PayerRegisteredBank;
                            invoice.PayerBankAccount = apply.PayerBankAccount;
                            invoice.OperatorName = apply.OperatorName;
                            invoice.Remark = apply.Remark;
                            invoice.Summary = "OA回传发票信息!";

                            invoice.Metadata = new Dictionary<string, string>();
                            invoice.Metadata["PayeeOperatorName"] = apply.Metadata.ContainsKey("PayeeOperatorName") ? apply.Metadata["PayeeOperatorName"] : "";
                            invoice.Metadata["ReviewerName"] = apply.Metadata.ContainsKey("ReviewerName") ? apply.Metadata["ReviewerName"] : "";
                            insertInvoices.Add(invoice);
                            //conn.Insert(invoice);

                            //foreach (var infoDetail in info.Details)
                            //{
                            var detail = new InvoiceDetail();
                            //detail.PopulateInstance(infoDetail);
                            detail.InvoiceKvid = invoiceKvid;
                            detail.TaxRate = invoice.TaxRate;
                            detail.GoodsFullName = "发票";
                            detail.AmountUntaxed = invoice.AmountUntaxed;
                            detail.Amount = invoice.Amount;
                            detail.AmountTax = invoice.AmountTax;
                            detail.Quantity = 1;
                            detail.QuantityUnitPrice = Math.Round(detail.Amount / detail.Quantity, 2);
                            detail.QuantityUnitPriceUntaxed = Math.Round(detail.AmountUntaxed / detail.Quantity, 2);
                            insertInvoiceDetails.Add(detail);
                            //}
                            #endregion

                            #region 更新申请状态
                            if (!applyRelations.IsNullOrEmpty())
                            {
                                foreach (var item in applyRelations)
                                {
                                    //无需更新
                                    if (item.AmountInvoice >= item.Amount) continue;
                                    var relationAmount = item.Amount - item.AmountInvoice;
                                    var amount = relationAmount <= invoice.Amount ? relationAmount : invoice.Amount;
                                    item.AmountInvoice += amount;
                                    item.AddOnlyProperties(o => o.AmountInvoice);
                                    if (item.AmountInvoice == item.Amount)
                                    {
                                        item.Status = (int)InvoiceApplyStatus.Completed;
                                        item.AddOnlyProperties(o => o.Status);
                                    }
                                    updateApplys.Add(item);
                                    //conn.UpdateOnly(item);
                                }
                            }
                            apply.AmountInvoice += invoice.Amount;
                            apply.AddOnlyProperties(o => o.AmountInvoice);
                            if (apply.AmountInvoice == apply.Amount)
                            {
                                apply.Status = (int)InvoiceApplyStatus.Completed;
                                apply.AddOnlyProperties(o => o.Status);
                            }
                            updateApplys.Add(apply);
                            //conn.UpdateOnly(apply);
                            #endregion
                        }

                        #region 收款与发票关联
                        var payments = conn.Select<Payment>(o => Sql.In(o.Kvid, applyRelations.ConvertAll(p => p.BizKvid)));//Type:Bank,Split,Pos,Cash,WeChat...
                        List<Payment> parentPayments = new List<Payment>();//也要更新父级的AmountInvoice
                        if (!payments.IsNullOrEmpty())
                        {
                            var correlatedInvoices = insertInvoices.Correlating(payments);
                            insertInvoices.AddRange(correlatedInvoices);
                            var currentRootKvids = payments.Where(o => o.Type == PaymentType.Split).ToList().ConvertAll(p => p.RootKvid);
                            if (!currentRootKvids.IsNullOrEmpty()) parentPayments = conn.Select<Payment>(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Kvid, currentRootKvids));
                        }
                        #endregion

                        if (!updateApplys.IsNullOrEmpty())
                        {
                            updateApplys.ForEach(o => conn.UpdateOnly(o));
                            rtns.Results.AddRange(updateApplys);
                        }
                        if (!insertInvoices.IsNullOrEmpty())
                        {
                            foreach (var item in insertInvoices)
                            {
                                if (item.Type == "Relation") item.SerialNumber = item.GetSubSerialNumber(conn);
                                conn.Insert(item);
                            }
                        }
                        if (!insertInvoiceDetails.IsNullOrEmpty()) insertInvoiceDetails.ForEach(o => conn.Insert(o));

                        if (!payments.IsNullOrEmpty())
                        {
                            payments.ForEach(o => conn.UpdateOnly(o));
                        }
                        if (!parentPayments.IsNullOrEmpty())
                        {
                            parentPayments.ForEach(o => o.RecalculateAmountInvoice(conn));
                        }
                    }

                    trans.Commit();
                }
                catch (Exception ex)
                {
                    trans.Rollback();
                    throw ex;
                }
            }
            else
            {
                var trans = conn.OpenTransaction();
                try
                {
                    foreach (var apply in applys)
                    {
                        if (apply.Metadata.IsNullOrEmpty()) apply.Metadata = new Dictionary<string, string>();
                        apply.Metadata["SeeyonResult"] = Result;
                        apply.Metadata["SeeyonMessage"] = Message;
                        apply.AddOnlyProperties(o => o.Metadata);
                        apply.Status = (int)InvoiceApplyStatus.ApplyReject;
                        apply.AddOnlyProperties(o => o.Status);
                        conn.UpdateOnly(apply);
                        rtns.Results.Add(apply);
                        var log = new EntityLog<InvoiceApply>();
                        log.OwnerKvid = apply.Kvid;
                        log.Title = "账单借票";
                        log.Type = "OA申请";
                        log.Summary = "OA审批不通过,驳回了账单借票审批";
                        log.Remark = Message;
                        log.CreatorName = "OA审批";
                        conn.Insert(log);
                    }
                    trans.Commit();
                }
                catch (Exception ex)
                {
                    trans.Rollback();
                    throw ex;
                }
            }

            return rtns;
        }
    }
}
